@@ -1,11 +1,12 @@
 /* Pango
  * syriac-fc.h:
  *
- * Copyright (C) 2000, 2003 Red Hat Software
- * Author: Owen Taylor <otaylor@redhat.com>
- *
+ * Copyright (C) 2000, 2003, 2007 Red Hat Software
  * Copyright (C) 2004 Emil Soleyman-Zomalan
- * Author: Emil Soleyman-Zomalan <emil@soleyman.com>
+ * Authors:
+ *   Owen Taylor <otaylor@redhat.com>
+ *   Emil Soleyman-Zomalan <emil@soleyman.com>
+ *   Behdad Esfahbod <behdad@behdad.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
 #include "syriac-ot.h"
@@ -77,7 +78,7 @@ static const PangoOTFeatureMap gpos_features[] =
 };
 
 static void
-syriac_engine_shape (PangoEngineShape *engine,
+syriac_engine_shape (PangoEngineShape *engine G_GNUC_UNUSED,
 		     PangoFont        *font,
 		     const char       *text,
 		     gint              length,
@@ -106,20 +107,6 @@ syriac_engine_shape (PangoEngineShape *engine,
   if (!face)
     return;
 
-  desc.script = analysis->script;
-  desc.language = analysis->language;
-
-  desc.n_static_gsub_features = G_N_ELEMENTS (gsub_features);
-  desc.static_gsub_features = gsub_features;
-  desc.n_static_gpos_features = G_N_ELEMENTS (gpos_features);
-  desc.static_gpos_features = gpos_features;
-
-  /* TODO populate other_features from analysis->extra_attrs */
-  desc.n_other_features = 0;
-  desc.other_features = NULL;
-
-  ruleset = pango_ot_ruleset_get_for (pango_ot_info_get (face), &desc);
-
   buffer = pango_ot_buffer_new (fc_font);
   pango_ot_buffer_set_rtl (buffer, analysis->level % 2 != 0);
   pango_ot_buffer_set_zero_width_marks (buffer, TRUE);
@@ -164,6 +151,20 @@ syriac_engine_shape (PangoEngineShape *engine,
 
   g_free (properties);
 
+  desc.script = analysis->script;
+  desc.language = analysis->language;
+
+  desc.n_static_gsub_features = G_N_ELEMENTS (gsub_features);
+  desc.static_gsub_features = gsub_features;
+  desc.n_static_gpos_features = G_N_ELEMENTS (gpos_features);
+  desc.static_gpos_features = gpos_features;
+
+  /* TODO populate other_features from analysis->extra_attrs */
+  desc.n_other_features = 0;
+  desc.other_features = NULL;
+
+  ruleset = pango_ot_ruleset_get_for_description (pango_ot_info_get (face), &desc);
+
   pango_ot_ruleset_substitute (ruleset, buffer);
   pango_ot_ruleset_position (ruleset, buffer);
   pango_ot_buffer_output (buffer, glyphs);
