@@ -19,12 +19,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
-#include "pango-glyph-item-private.h"
+#include "pango-glyph-item.h"
 #include "pango-layout-private.h"
 #include "pango-engine-private.h"
+#include "pango-impl-utils.h"
 
 typedef struct _EllipsizeState EllipsizeState;
 typedef struct _RunInfo        RunInfo;
@@ -114,7 +115,7 @@ init_state (EllipsizeState  *state,
 	    PangoAttrList   *attrs)
 {
   GSList *l;
-  int i, j;
+  int i;
   int start_offset;
 
   state->layout = line->layout;
@@ -123,18 +124,14 @@ init_state (EllipsizeState  *state,
   state->n_runs = g_slist_length (line->runs);
   state->run_info = g_new (RunInfo, state->n_runs);
 
-  start_offset = g_utf8_strlen (line->layout->text,
+  start_offset = pango_utf8_strlen (line->layout->text,
 				line->start_index);
 
   state->total_width = 0;
   for (l = line->runs, i = 0; l; l = l->next, i++)
     {
       PangoGlyphItem *run = l->data;
-      int width = 0;
-
-      for (j = 0; j < run->glyphs->num_glyphs; j++)
-	width += run->glyphs->glyphs[j].geometry.width;
-
+      int width = pango_glyph_string_get_width (run->glyphs);
       state->run_info[i].run = run;
       state->run_info[i].width = width;
       state->run_info[i].start_offset = start_offset;
@@ -191,16 +188,16 @@ static gboolean
 line_iter_next_cluster (EllipsizeState *state,
 			LineIter       *iter)
 {
-  if (!_pango_glyph_item_iter_next_cluster (&iter->run_iter))
+  if (!pango_glyph_item_iter_next_cluster (&iter->run_iter))
     {
       if (iter->run_index == state->n_runs - 1)
 	return FALSE;
       else
 	{
 	  iter->run_index++;
-	  _pango_glyph_item_iter_init_start (&iter->run_iter,
-					     state->run_info[iter->run_index].run,
-					     state->layout->text);
+	  pango_glyph_item_iter_init_start (&iter->run_iter,
+					    state->run_info[iter->run_index].run,
+					    state->layout->text);
 	}
     }
 
@@ -213,16 +210,16 @@ static gboolean
 line_iter_prev_cluster (EllipsizeState *state,
 			LineIter       *iter)
 {
-  if (!_pango_glyph_item_iter_prev_cluster (&iter->run_iter))
+  if (!pango_glyph_item_iter_prev_cluster (&iter->run_iter))
     {
       if (iter->run_index == 0)
 	return FALSE;
       else
 	{
 	  iter->run_index--;
-	  _pango_glyph_item_iter_init_end (&iter->run_iter,
-					   state->run_info[iter->run_index].run,
-					   state->layout->text);
+	  pango_glyph_item_iter_init_end (&iter->run_iter,
+					  state->run_info[iter->run_index].run,
+					  state->layout->text);
 	}
     }
 
@@ -517,9 +514,9 @@ find_initial_span (EllipsizeState *state)
   glyph_item = state->run_info[i].run;
 
   cluster_width = 0;		/* Quiet GCC, the line must have at least one cluster */
-  for (have_cluster = _pango_glyph_item_iter_init_start (run_iter, glyph_item, state->layout->text);
+  for (have_cluster = pango_glyph_item_iter_init_start (run_iter, glyph_item, state->layout->text);
        have_cluster;
-       have_cluster = _pango_glyph_item_iter_next_cluster (run_iter))
+       have_cluster = pango_glyph_item_iter_next_cluster (run_iter))
     {
       cluster_width = get_cluster_width (&state->gap_start_iter);
 
@@ -640,7 +637,7 @@ fixup_ellipsis_run (EllipsizeState *state)
   /* Fix up the item to point to the entire elided text */
   item->offset = state->gap_start_iter.run_iter.start_index;
   item->length = state->gap_end_iter.run_iter.end_index - item->offset;
-  item->num_chars = g_utf8_strlen (state->layout->text + item->offset, item->length);
+  item->num_chars = pango_utf8_strlen (state->layout->text + item->offset, item->length);
 
   /* The level for the item is the minimum level of the elided text */
   level = G_MAXINT;
@@ -727,22 +724,16 @@ current_width (EllipsizeState *state)
  **/
 gboolean
 _pango_layout_line_ellipsize (PangoLayoutLine *line,
-			      PangoAttrList   *attrs)
+			      PangoAttrList   *attrs,
+			      int              goal_width)
 {
   EllipsizeState state;
-  int goal_width;
   gboolean is_ellipsized = FALSE;
 
-  if (line->layout->ellipsize == PANGO_ELLIPSIZE_NONE ||
-      line->layout->width < 0)
-    goto ret;
+  g_return_val_if_fail (line->layout->ellipsize != PANGO_ELLIPSIZE_NONE && goal_width >= 0, is_ellipsized);
 
   init_state (&state, line, attrs);
 
-  goal_width = state.layout->width;
-  if (state.layout->indent > 0 && state.layout->alignment != PANGO_ALIGN_CENTER)
-    goal_width -= state.layout->indent;
-
   if (state.total_width <= goal_width)
     goto out;
 
@@ -762,6 +753,6 @@ _pango_layout_line_ellipsize (PangoLayoutLine *line,
 
  out:
   free_state (&state);
- ret:
+
   return is_ellipsized;
 }
