@@ -177,7 +177,7 @@ get_glyph_class (gunichar   charcode,
       (charcode >= 0xFE70 && charcode <= 0XFEFF))   /* Arabic Presentation Forms-B */
     return FALSE;
 
-  switch (g_unichar_type (charcode))
+  switch ((int) g_unichar_type (charcode))
     {
     case G_UNICODE_COMBINING_MARK:
     case G_UNICODE_ENCLOSING_MARK:
@@ -298,7 +298,7 @@ pango_ot_info_get_gdef (PangoOTInfo *info)
 	  error = HB_Load_GDEF_Table (info->face, &info->gdef);
 
 	  if (error && error != HB_Err_Not_Covered)
-	    g_warning ("Error loading GDEF table 0x%04x", error);
+	    g_warning ("Error loading GDEF table 0x%04X", error);
 
 	  if (!info->gdef)
 	    error = HB_New_GDEF_Table (&info->gdef);
@@ -328,7 +328,7 @@ pango_ot_info_get_gsub (PangoOTInfo *info)
 	  error = HB_Load_GSUB_Table (info->face, &info->gsub, gdef);
 
 	  if (error && error != HB_Err_Not_Covered)
-	    g_warning ("Error loading GSUB table 0x%04x", error);
+	    g_warning ("Error loading GSUB table 0x%04X", error);
 	}
     }
 
@@ -352,7 +352,7 @@ pango_ot_info_get_gpos (PangoOTInfo *info)
 	  error = HB_Load_GPOS_Table (info->face, &info->gpos, gdef);
 
 	  if (error && error != HB_Err_Not_Covered)
-	    g_warning ("Error loading GPOS table 0x%04x", error);
+	    g_warning ("Error loading GPOS table 0x%04X", error);
 	}
     }
 
@@ -522,7 +522,6 @@ pango_ot_info_find_language (PangoOTInfo      *info,
     return FALSE;
 
   g_return_val_if_fail (script_index < script_list->ScriptCount, FALSE);
-
   script = &script_list->ScriptRecord[script_index].Script;
 
   for (i = 0; i < script->LangSysCount; i++)
@@ -610,7 +609,6 @@ pango_ot_info_find_feature  (PangoOTInfo      *info,
     return FALSE;
 
   g_return_val_if_fail (script_index < script_list->ScriptCount, FALSE);
-
   script = &script_list->ScriptRecord[script_index].Script;
 
   if (language_index == PANGO_OT_DEFAULT_LANGUAGE)
@@ -625,7 +623,8 @@ pango_ot_info_find_feature  (PangoOTInfo      *info,
     {
       HB_UShort index = lang_sys->FeatureIndex[i];
 
-      if (feature_list->FeatureRecord[index].FeatureTag == feature_tag)
+      if (index < feature_list->FeatureCount &&
+	  feature_list->FeatureRecord[index].FeatureTag == feature_tag)
 	{
 	  if (feature_index)
 	    *feature_index = index;
@@ -706,7 +705,6 @@ pango_ot_info_list_languages (PangoOTInfo      *info,
     return NULL;
 
   g_return_val_if_fail (script_index < script_list->ScriptCount, NULL);
-
   script = &script_list->ScriptRecord[script_index].Script;
 
   result = g_new (PangoOTTag, script->LangSysCount + 1);
@@ -748,7 +746,7 @@ pango_ot_info_list_features  (PangoOTInfo      *info,
   HB_ScriptTable *script;
   HB_LangSys *lang_sys;
 
-  int i;
+  int i, j;
 
   g_return_val_if_fail (PANGO_IS_OT_INFO (info), NULL);
 
@@ -763,7 +761,6 @@ pango_ot_info_list_features  (PangoOTInfo      *info,
     return NULL;
 
   g_return_val_if_fail (script_index < script_list->ScriptCount, NULL);
-
   script = &script_list->ScriptRecord[script_index].Script;
 
   if (language_index == PANGO_OT_DEFAULT_LANGUAGE)
@@ -776,14 +773,16 @@ pango_ot_info_list_features  (PangoOTInfo      *info,
 
   result = g_new (PangoOTTag, lang_sys->FeatureCount + 1);
 
+  j = 0;
   for (i = 0; i < lang_sys->FeatureCount; i++)
     {
       HB_UShort index = lang_sys->FeatureIndex[i];
 
-      result[i] = feature_list->FeatureRecord[index].FeatureTag;
+      if (index < feature_list->FeatureCount)
+	result[j++] = feature_list->FeatureRecord[index].FeatureTag;
     }
 
-  result[i] = 0;
+  result[j] = 0;
 
   return result;
 }
