@@ -19,13 +19,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <stdlib.h>
 #include "config.h"
 
 #include <glib.h>
 #include <gmodule.h>
 #include "pango-break.h"
 #include "pango-context.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 #include "pango-engine.h"
 #include "pango-enum-types.h"
 
@@ -36,12 +37,12 @@
 #endif
 #include <glib/gprintf.h>
 
-#if USE_LA_MODULES
+#ifdef USE_LA_MODULES
 #define SOEXT ".la"
 #else
 #define SOEXT ("." G_MODULE_SUFFIX)
 #endif
-#define SOEXT_LEN (strlen (SOEXT))
+#define SOEXT_LEN ((int) strlen (SOEXT))
 
 static gboolean
 string_needs_escape (const char *str)
@@ -65,7 +66,7 @@ escape_string (const char *str)
   while (TRUE)
     {
       char c = *str++;
-      
+
       switch (c)
 	{
 	case '\0':
@@ -98,7 +99,7 @@ string_from_script (PangoScript script)
   GEnumValue *value;
   if (!class)
     class = g_type_class_ref (PANGO_TYPE_SCRIPT);
-  
+
   value = g_enum_get_value (class, script);
   if (!value)
     {
@@ -109,7 +110,7 @@ string_from_script (PangoScript script)
   return value->value_nick;
 }
 
-static void 
+static void
 query_module (const char *dir, const char *name)
 {
   void (*list) (PangoEngineInfo **engines, gint *n_engines);
@@ -124,12 +125,12 @@ query_module (const char *dir, const char *name)
     path = g_strdup (name);
   else
     path = g_build_filename (dir, name, NULL);
-  
-  module = g_module_open (path, G_MODULE_BIND_LAZY);
+
+  module = g_module_open (path, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
 
   if (!module)
     g_printerr ("Cannot load module %s: %s\n", path, g_module_error ());
-	  
+
   if (module &&
       GET_SYMBOL (module, "script_engine_list", list) &&
       GET_SYMBOL (module, "script_engine_init", init) &&
@@ -157,7 +158,7 @@ query_module (const char *dir, const char *name)
 	      quote = "";
 	      quoted_path = g_strdup (path);
 	    }
-	  
+
 	  g_printf ("%s%s%s %s %s %s", quote, quoted_path, quote,
 		    engines[i].id, engines[i].engine_type, engines[i].render_type);
 	  g_free (quoted_path);
@@ -175,11 +176,22 @@ query_module (const char *dir, const char *name)
     {
       g_printerr ("%s does not export Pango module API\n", path);
     }
-  
+
   g_free (path);
   if (module)
     g_module_close (module);
-}		       
+}
+
+static gboolean
+show_version(const char *name G_GNUC_UNUSED,
+	     const char *arg G_GNUC_UNUSED,
+	     gpointer    data G_GNUC_UNUSED,
+	     GError    **error G_GNUC_UNUSED)
+{
+  g_printf("pango-querymodules (%s) %s\n", PACKAGE_NAME, PACKAGE_VERSION);
+  g_printf("module interface version: %s\n", MODULE_VERSION);
+  exit(0);
+}
 
 int
 main (int argc, char **argv)
@@ -187,9 +199,35 @@ main (int argc, char **argv)
   char *cwd;
   int i;
   char *path;
+  GOptionContext *context;
+  GError *parse_error = NULL;
+  GOptionEntry entries[] =
+    {
+      {"version",	0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, &show_version,
+       "Show version numbers",                                             NULL},
+      {NULL}
+    };
+
+  context = g_option_context_new ("- [MODULE]...");
+  g_option_context_add_main_entries (context, entries, NULL);
+
+  if (!g_option_context_parse (context, &argc, &argv, &parse_error))
+    {
+      if (parse_error != NULL)
+	{
+	  g_printerr("Parse option error: %s\n", parse_error->message);
+	}
+      else
+	{
+	  g_printerr("Parse option error\n");
+	}
+      exit(1);
+    }
+
+  g_option_context_free(context);
 
   g_type_init ();
-  
+
   g_printf ("# Pango Modules file\n"
 	    "# Automatically generated file, do not edit\n"
 	    "#\n");
@@ -198,7 +236,7 @@ main (int argc, char **argv)
     {
       char **dirs;
       int i;
-      
+
       path = pango_config_key_get ("Pango/ModulesPath");
       if (!path)
 	path = g_build_filename (pango_get_lib_subdirectory (),
@@ -225,7 +263,7 @@ main (int argc, char **argv)
 		  if (len > SOEXT_LEN && strcmp (dent + len - SOEXT_LEN, SOEXT) == 0)
 		    query_module (dirs[i], dent);
 		}
-	      
+
 	      g_dir_close (dir);
 	    }
 	}
@@ -235,12 +273,12 @@ main (int argc, char **argv)
   else
     {
       cwd = g_get_current_dir ();
-      
+
       for (i=1; i<argc; i++)
 	query_module (cwd, argv[i]);
 
       g_free (cwd);
     }
-  
+
   return 0;
 }
